=== modified file 'src/mir_screen.cpp'
--- src/mir_screen.cpp	2016-01-26 15:01:31 +0000
+++ src/mir_screen.cpp	2016-01-28 10:38:08 +0000
@@ -30,6 +30,7 @@
 #include <cstdio>
 #include <sstream>
 
+#include <assert.h>
 #include "screen_hardware.h"
 #include "power_state_change_reason.h"
 #include "server.h"
@@ -59,14 +60,17 @@
 
     void lock() override
     {
+        guard_lock = std::unique_lock<std::mutex>{mir_screen->guard};
     }
 
     void unlock() override
     {
+        guard_lock.unlock();
     }
     
 protected:
     MirScreen* const mir_screen;
+    std::unique_lock<std::mutex> guard_lock;
 };
 
 class usc::MirScreen::PowerOffLockableCallback : public usc::MirScreen::LockableCallback
@@ -75,7 +79,11 @@
 
     void operator()() override
     {
-        mir_screen->power_off_alarm_notification();
+        // We need to hold the lock before calling power_off_alarm_notification_l()
+        // (and not acquire it in that function) as we override the function to
+        // test for deadlock conditions. C.f. test_deadlock_lp1491566.cpp
+        assert(guard_lock.owns_lock());
+        mir_screen->power_off_alarm_notification_l();
     }
 };
 
@@ -85,7 +93,11 @@
 
     void operator()() override
     {
-        mir_screen->dimmer_alarm_notification();
+        // We need to hold the lock before calling dimmer_alarm_notification_l()
+        // (and not acquire it in that function) as we override the function to
+        // test for deadlock conditions. C.f. test_deadlock_lp1491566.cpp
+        assert(guard_lock.owns_lock());
+        mir_screen->dimmer_alarm_notification_l();
     }
 };
 
@@ -422,16 +434,14 @@
     return true;
 }
 
-void usc::MirScreen::power_off_alarm_notification()
+void usc::MirScreen::power_off_alarm_notification_l()
 {
-    std::lock_guard<std::mutex> lock{guard};
     configure_display_l(MirPowerMode::mir_power_mode_off, PowerStateChangeReason::inactivity);
     next_power_off = {};
 }
 
-void usc::MirScreen::dimmer_alarm_notification()
+void usc::MirScreen::dimmer_alarm_notification_l()
 {
-    std::lock_guard<std::mutex> lock{guard};
     if (current_power_mode != MirPowerMode::mir_power_mode_off)
         screen_hardware->set_dim_backlight();
     next_dimming = {};

=== modified file 'src/mir_screen.h'
--- src/mir_screen.h	2016-01-25 18:02:52 +0000
+++ src/mir_screen.h	2016-01-28 10:13:34 +0000
@@ -76,8 +76,8 @@
 
 protected:
     // These are protected virtual because we need to override them in tests
-    virtual void power_off_alarm_notification();
-    virtual void dimmer_alarm_notification();
+    virtual void power_off_alarm_notification_l();
+    virtual void dimmer_alarm_notification_l();
 
 private:
     enum class ForceResetTimers { no, yes };

=== modified file 'tests/integration-tests/test_deadlock_lp1491566.cpp'
--- tests/integration-tests/test_deadlock_lp1491566.cpp	2015-11-12 12:53:39 +0000
+++ tests/integration-tests/test_deadlock_lp1491566.cpp	2016-01-28 10:13:34 +0000
@@ -80,16 +80,16 @@
 public:
     using usc::MirScreen::MirScreen;
 
-    void dimmer_alarm_notification() override
+    void dimmer_alarm_notification_l() override
     {
         before_dimmer_alarm_func();
-        usc::MirScreen::dimmer_alarm_notification();
+        usc::MirScreen::dimmer_alarm_notification_l();
     }
 
-    void power_off_alarm_notification() override
+    void power_off_alarm_notification_l() override
     {
         before_power_off_alarm_func();
-        usc::MirScreen::power_off_alarm_notification();
+        usc::MirScreen::power_off_alarm_notification_l();
     }
 
     std::function<void()> before_dimmer_alarm_func = []{};

